Package: geessbin
Type: Package
Title: Modified Generalized Estimating Equations for Binary Outcome
Version: 1.0.2
Authors@R: c(person("Ryota", "Ishii", email = "r.ishii0808@gmail.com",
                  role = c("aut", "cre")),
                 person("Tomohiro", "Ohigashi", role = "ctb"),
                 person("Kazushi", "Maruo", role = "ctb"),
                 person("Masahiko", "Gosho", role = "ctb"))
Maintainer: Ryota Ishii <r.ishii0808@gmail.com>
Description: Analyze small-sample clustered or longitudinal data with binary 
    outcome using modified generalized estimating equations (GEE) with 
    bias-adjusted covariance estimator. The package provides any combination of 
    three GEE methods and 12 covariance estimators.
Depends: R (>= 3.5.0)
Imports: MASS (>= 7.3-45)
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
URL: https://github.com/rtishii/geessbin
RoxygenNote: 7.3.2
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-08 06:13:35 UTC; rishii
Author: Ryota Ishii [aut, cre],
  Tomohiro Ohigashi [ctb],
  Kazushi Maruo [ctb],
  Masahiko Gosho [ctb]
Repository: CRAN
Date/Publication: 2026-05-08 06:41:16 UTC
Built: R 4.6.0; ; 2026-05-08 07:18:43 UTC; unix
