# Generated by roxygen2: do not edit by hand

S3method(coef,geessbin)
S3method(coef,summary.geessbin)
S3method(fitted,geessbin)
S3method(model.matrix,geessbin)
S3method(predict,geessbin)
S3method(print,geessbin)
S3method(print,summary.geessbin)
S3method(residuals,geessbin)
S3method(summary,geessbin)
S3method(vcov,geessbin)
export(geessbin)
export(geessbin_all)
export(sqrtmat)
importFrom(MASS,ginv)
importFrom(stats,.getXlevels)
importFrom(stats,binomial)
importFrom(stats,cov)
importFrom(stats,delete.response)
importFrom(stats,formula)
importFrom(stats,glm.fit)
importFrom(stats,model.extract)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.pass)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,terms)
